'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var styled = require('styled-components');
var styled__default = _interopDefault(styled);
var reactWindow = require('react-window');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

function isDocumentElement(el) {
    return (el === document.documentElement || el === document.body || el === window);
}
function getScrollTop(el) {
    return isDocumentElement(el) ? window.pageYOffset : el.scrollTop;
}
function scrollTo(el, top) {
    if (isDocumentElement(el)) {
        window.scrollTo(0, top);
    }
    else {
        el.scrollTop = top;
    }
}
function getScrollParent(el) {
    let style = getComputedStyle(el);
    const excludeStaticParent = (style.position === 'absolute');
    if (style.position === 'fixed') {
        return document.documentElement;
    }
    for (let parent = el; (parent = parent.parentElement);) {
        style = getComputedStyle(parent);
        if (excludeStaticParent && style.position === 'static') {
            continue;
        }
        else if (/(auto|scroll)/.test(`${style.overflow}${style.overflowY}${style.overflowX}`)) {
            return parent;
        }
    }
    return document.documentElement;
}
function smoothScrollTo(element, to, duration, callback) {
    const start = getScrollTop(element);
    const change = to - start;
    let currentTime = 0;
    function easeOutCubic(t, s, c, d) {
        return c * ((t = t / d - 1) * t * t + 1) + s;
    }
    function smoothScroller() {
        currentTime += 5;
        scrollTo(element, easeOutCubic(currentTime, start, change, duration));
        if (currentTime < duration) {
            requestAnimationFrame(smoothScroller);
        }
        else {
            callback && callback(element);
        }
    }
    requestAnimationFrame(smoothScroller);
}
function isPrimitive(test) {
    return (test !== Object(test));
}
function isArrayWithLength(test) {
    return Array.isArray(test) && !!test.length;
}
function isTouchDevice() {
    return window.matchMedia('(pointer: coarse)').matches;
}
function createClassName(classNames) {
    return classNames.filter(x => x).join(' ');
}
function isPlainObject(test) {
    return test && (typeof test === 'object') && !Array.isArray(test);
}
function createID(idPrefix, idSuffix) {
    if (!idPrefix) {
        return undefined;
    }
    if (!idSuffix) {
        return idPrefix;
    }
    return `${idPrefix}-${idSuffix}`;
}
function trimAndFormatFilterStr(value, filterIsCaseSensitive) {
    const formatVal = value.replace(/^\s+|\s+$/g, '');
    return !filterIsCaseSensitive ? formatVal.toLowerCase() : formatVal;
}
function mergeDeep(target, source) {
    const output = Object.assign({}, target);
    Object.keys(source).forEach((key) => {
        if (isPlainObject(source[key])) {
            output[key] = (!(key in target))
                ? source[key]
                : mergeDeep(target[key], source[key]);
        }
        else {
            output[key] = source[key];
        }
    });
    return output;
}
function scrollMenuIntoViewOnOpen(menuEl, scrollMenuIntoView, onMenuOpen) {
    if (!scrollMenuIntoView || !menuEl || !menuEl.getBoundingClientRect) {
        onMenuOpen && onMenuOpen();
        return;
    }
    const menuRect = menuEl.getBoundingClientRect();
    const viewHeight = window.innerHeight;
    if ((viewHeight - menuRect.top) >= menuRect.height) {
        onMenuOpen && onMenuOpen();
        return;
    }
    const scrollParent = getScrollParent(menuEl);
    const scrollTop = getScrollTop(scrollParent);
    const scrollSpaceBelow = (scrollParent.getBoundingClientRect().height - scrollTop - menuRect.top);
    if (scrollSpaceBelow < menuRect.height) {
        onMenuOpen && onMenuOpen();
        return;
    }
    const marginBottom = parseInt(getComputedStyle(menuEl).marginBottom || '0', 10);
    const scrollDown = (menuRect.bottom - viewHeight + scrollTop + marginBottom);
    smoothScrollTo(scrollParent, scrollDown, 300, onMenuOpen);
}
function validateSetValueOption(option, menuOptions, getOptionValueCB) {
    if (option === null || option === undefined || Array.isArray(option)) {
        return undefined;
    }
    const optionValue = (option && isPrimitive(option)) ? option : getOptionValueCB(option);
    return menuOptions.find((mOption) => mOption.value === optionValue) || undefined;
}
const renderControlEmphasis = (boxShadow, borderColor, invalidColor, invalidFocus, boxShadowColor, focusedBorderColor, isFocused, isInvalid) => {
    if (isFocused) {
        return styled.css `
      border-color: ${isInvalid ? invalidColor : focusedBorderColor};
      box-shadow: ${boxShadow} ${isInvalid ? invalidFocus : boxShadowColor};
    `;
    }
    return styled.css `
    border-color: ${isInvalid ? invalidColor : borderColor};
  `;
};

const mergeThemeStyles = (themeConfig) => {
    return !isPlainObject(themeConfig)
        ? Object.assign({}, defaultTheme) : mergeDeep(defaultTheme, themeConfig);
};
const defaultTheme = {
    color: {
        accent: '#007bff',
        border: '#ced4da',
        invalid: '#dc3545',
        disabled: '#e9ecef',
        placeholder: '#6E7276',
        invalidFocus: 'rgba(220, 53, 69, 0.25)',
    },
    select: {},
    loader: {
        opacity: '0.42',
        size: '0.625rem',
        color: '#007bff',
        padding: '0.375rem 0.75rem',
    },
    icon: {
        color: '#cccccc',
        padding: '0.5rem 0.9375rem',
        hoverColor: '#A6A6A6',
        clear: {
            fontWeight: 900,
            fontSize: '0.85em',
            transition: 'color 0.15s ease-in-out',
            fontFamily: '"Helvetica", "Arial", sans-serif',
        },
        caret: {
            size: '7px',
            transition: 'transform 0.225s ease-in-out, color 0.15s ease-in-out',
        },
    },
    control: {
        borderWidth: '1px',
        borderStyle: 'solid',
        borderRadius: '0.25rem',
        boxShadow: '0 0 0 0.2rem',
        padding: '0.375rem 0.75rem',
        height: 'calc(1.5em + 0.75rem + 2px)',
        boxShadowColor: 'rgba(0, 123, 255, 0.25)',
        focusedBorderColor: 'rgba(0, 123, 255, 0.75)',
        transition: 'box-shadow 0.15s ease-in-out, border-color 0.15s ease-in-out',
    },
    menu: {
        width: '100%',
        margin: '0.5rem 0',
        padding: '0.15rem 0',
        borderRadius: '0.25rem',
        backgroundColor: '#fff',
        boxShadow: '0 0 0 1px rgba(0, 0, 0, 0.1), 0 4px 11px rgba(0, 0, 0, 0.1)',
        option: {
            textAlign: 'left',
            selectedColor: '#fff',
            selectedBgColor: '#007bff',
            padding: '0.375rem 0.75rem',
            focusedBgColor: 'rgba(0, 123, 255, 0.20)',
        },
    },
    noOptions: {
        fontSize: '1.25rem',
        margin: '0.25rem 0',
        color: 'hsl(0, 0%, 60%)',
        padding: '0.375rem 0.75rem',
    },
};

const useTheme = (themeConfig) => {
    const [theme, setTheme] = React.useState(() => mergeThemeStyles(themeConfig));
    React.useEffect(() => {
        const mergedStyles = mergeThemeStyles(themeConfig);
        setTheme(mergedStyles);
    }, [themeConfig]);
    return theme;
};

const useDebounce = (value, delay) => {
    const [debouncedValue, setDebouncedValue] = React.useState(value);
    React.useEffect(() => {
        if (delay === undefined) {
            return;
        }
        const handler = setTimeout(() => {
            setDebouncedValue(value);
        }, delay);
        return () => {
            clearTimeout(handler);
        };
    }, [value, delay]);
    return (delay === undefined) ? value : debouncedValue;
};

const MENU_ITEM_SIZE_DEFAULT = 35;
const MENU_MAX_HEIGHT_DEFAULT = 300;
const NO_OPTIONS_MSG_DEFAULT = 'No options';
const PLACEHOLDER_DEFAULT = 'Select option..';
const OPTIONS_DEFAULT = [];
const NO_SELECTED_OPTION = {};
const FOCUSED_OPTION_DEFAULT = {
    index: -1,
};
const SELECTED_OPTION_DEFAULT = {
    initFlag: true,
};

const useMenuOptions = (options, debouncedInputValue, getOptionValueCB, getOptionLabelCB, getIsOptionDisabledCB, getFilterOptionStringCB, filterIsCaseSensitive) => {
    const [menuOptions, setMenuOptions] = React.useState(OPTIONS_DEFAULT);
    React.useEffect(() => {
        const createMenuOptions = () => {
            const cleanSearchInputValue = trimAndFormatFilterStr(debouncedInputValue, filterIsCaseSensitive);
            const optionSatisfiesFilter = (menuOption) => {
                const cleanStringifiedOption = trimAndFormatFilterStr(getFilterOptionStringCB(menuOption), filterIsCaseSensitive);
                return cleanStringifiedOption.indexOf(cleanSearchInputValue) > -1;
            };
            const parseMenuOption = (data) => {
                const menuOption = {
                    data,
                    label: getOptionLabelCB(data),
                    value: getOptionValueCB(data),
                };
                if (cleanSearchInputValue && !optionSatisfiesFilter(menuOption)) {
                    return undefined;
                }
                return Object.assign(Object.assign({}, menuOption), (getIsOptionDisabledCB(data) && { isDisabled: true }));
            };
            return options.reduce((accumulator, data) => {
                const option = parseMenuOption(data);
                option && accumulator.push(option);
                return accumulator;
            }, []);
        };
        setMenuOptions(createMenuOptions() || OPTIONS_DEFAULT);
    }, [options, filterIsCaseSensitive, debouncedInputValue, getFilterOptionStringCB, getOptionValueCB, getOptionLabelCB, getIsOptionDisabledCB]);
    return menuOptions;
};

const fadeIn = styled.keyframes `
  from {
    opacity: 0;
  } to {
    opacity: 1;
  }
`;
const bounce = styled.keyframes `
  0%, 80%, 100% {
    transform: scale(0);
  } 40% {
    transform: scale(1.0);
  }
`;
const bounceAnimationCss = styled.css `
  animation: ${bounce} 1.19s ease-in-out infinite;
`;
const fadeInAnimationCss = styled.css `
  animation: ${fadeIn} 0.225s ease-in-out forwards;
`;
const mediaQueryIsIE = '@media all and (-ms-high-contrast: none), (-ms-high-contrast: active)';

const attrPrefix = 'rfs';
const isNotProduction = (process.env.NODE_ENV !== 'production');
const OPTION_CLS = `${attrPrefix}-option`;
const OPTION_FOCUSED_CLS = `${OPTION_CLS}-focused`;
const OPTION_SELECTED_CLS = `${OPTION_CLS}-selected`;
const OPTION_DISABLED_CLS = `${OPTION_CLS}-disabled`;
const CARET_ICON_CLS = `${attrPrefix}-caret-icon`;
const CLEAR_ICON_CLS = `${attrPrefix}-clear-icon`;
const AUTOSIZE_INPUT_CLS = `${attrPrefix}-autosize-input`;
const MENU_CONTAINER_CLS = `${attrPrefix}-menu-container`;
const SELECT_CONTAINER_CLS = `${attrPrefix}-select-container`;
const CONTROL_CONTAINER_CLS = `${attrPrefix}-control-container`;
const CLEAR_ICON_TESTID = isNotProduction ? CLEAR_ICON_CLS : undefined;
const AUTOSIZE_INPUT_TESTID = isNotProduction ? AUTOSIZE_INPUT_CLS : undefined;
const MENU_CONTAINER_TESTID = isNotProduction ? MENU_CONTAINER_CLS : undefined;
const SELECT_CONTAINER_TESTID = isNotProduction ? SELECT_CONTAINER_CLS : undefined;
const CONTROL_CONTAINER_TESTID = isNotProduction ? CONTROL_CONTAINER_CLS : undefined;

const Option = React__default.memo(({ index, style, data: { idSuffix, menuOptions, selectOption, renderOptionLabel, focusedOptionIndex, selectedOptionValue, }, }) => {
    const { data, value, label, isDisabled } = menuOptions[index];
    const isSelected = (value === selectedOptionValue);
    const className = createClassName([
        OPTION_CLS,
        isDisabled && OPTION_DISABLED_CLS,
        isSelected && OPTION_SELECTED_CLS,
        (index === focusedOptionIndex) && OPTION_FOCUSED_CLS
    ]);
    return (React__default.createElement("div", { style: style, className: className, id: idSuffix ? createID(`${OPTION_CLS}-${index}`, idSuffix) : undefined, onClick: !isDisabled ? (() => selectOption({ data, value, label }, isSelected)) : undefined }, renderOptionLabel(data)));
}, reactWindow.areEqual);
Option.displayName = 'Option';

const NoOptionsMsg = styled__default.div `
  text-align: center;
  color: ${({ theme }) => theme.noOptions.color};
  margin: ${({ theme }) => theme.noOptions.margin};
  padding: ${({ theme }) => theme.noOptions.padding};
  font-size: ${({ theme }) => theme.noOptions.fontSize};
`;
const Menu = React__default.forwardRef(({ width, itemSize, idSuffix, maxHeight, menuOptions, selectOption, noOptionsMsg, overscanCount, renderOptionLabel, focusedOptionIndex, selectedOptionValue, }, ref) => {
    const itemData = React.useMemo(() => ({
        idSuffix,
        menuOptions,
        selectOption,
        renderOptionLabel,
        focusedOptionIndex,
        selectedOptionValue,
    }), [idSuffix, menuOptions, focusedOptionIndex, selectedOptionValue, selectOption, renderOptionLabel]);
    if (!isArrayWithLength(menuOptions)) {
        return (React__default.createElement(NoOptionsMsg, null, noOptionsMsg));
    }
    return (React__default.createElement(reactWindow.FixedSizeList, { ref: ref, width: width, itemSize: itemSize, itemData: itemData, overscanCount: overscanCount, itemCount: menuOptions.length, height: Math.min(maxHeight, menuOptions.length * itemSize) }, Option));
});
Menu.displayName = 'Menu';

const _valueDivCss = styled.css `
  top: 50%;
  overflow: hidden;
  position: absolute;
  white-space: nowrap;
  box-sizing: border-box;
  text-overflow: ellipsis;
  transform: translateY(-50%);
`;
const StyledValue = styled__default.div `
  ${_valueDivCss}
  max-width: calc(100% - 0.5rem);
`;
const Placeholder = styled__default.div `
  ${_valueDivCss}
  color: ${({ theme }) => theme.color.placeholder};
`;
const Value = React__default.memo(({ inputValue, placeholder, renderOptionLabel, selectedOptionData, }) => {
    if (inputValue) {
        return null;
    }
    if (!selectedOptionData) {
        return (React__default.createElement(Placeholder, null, placeholder));
    }
    return (React__default.createElement(StyledValue, null, renderOptionLabel(selectedOptionData)));
});
Value.displayName = 'Value';

const _inputMinWidthPx = 2;
const _wrapperDivStyle = {
    display: 'inline-block'
};
const _inputStaticAttributes = {
    type: 'text',
    spellCheck: false,
    autoCorrect: 'off',
    autoComplete: 'off',
    autoCapitalize: 'none',
    'data-testid': AUTOSIZE_INPUT_TESTID,
};
const StyledSizer = styled__default.div `
  top: 0;
  left: 0;
  height: 0;
  overflow: scroll;
  white-space: pre;
  position: absolute;
  visibility: hidden;
  font-size: inherit;
  font-weight: inherit;
  font-family: inherit;
`;
const StyledAutosizeInput = styled__default.input `
  border: 0;
  outline: 0;
  padding: 0;
  cursor: text;
  background: 0;
  color: inherit;
  font-size: inherit;
  font-weight: inherit;
  font-family: inherit;
  box-sizing: content-box;

  :read-only {
    opacity: 0;
    cursor: default;
  }

  ${mediaQueryIsIE} {
    ::-ms-clear { 
      display: none; 
    }
  }
`;
const AutosizeInput = React__default.memo(React__default.forwardRef(({ id, onBlur, onFocus, onChange, disabled, ariaLabel, inputValue, isSearchable, addClassNames, ariaLabelledBy, }, ref) => {
    const sizerRef = React.useRef(null);
    const [inputWidth, setInputWidth] = React.useState(_inputMinWidthPx);
    const inputAriaAttributes = {
        'aria-label': ariaLabel,
        'aria-autocomplete': 'list',
        'aria-labelledby': ariaLabelledBy,
    };
    React.useEffect(() => {
        if (sizerRef.current) {
            setInputWidth(sizerRef.current.scrollWidth + _inputMinWidthPx);
        }
    }, [inputValue]);
    return (React__default.createElement("div", { style: _wrapperDivStyle },
        React__default.createElement(StyledAutosizeInput, Object.assign({ id: id, ref: ref, onBlur: onBlur, onFocus: onFocus, value: inputValue, disabled: disabled, readOnly: !isSearchable }, inputAriaAttributes, _inputStaticAttributes, { style: { width: inputWidth }, onChange: isSearchable ? onChange : undefined, className: addClassNames ? AUTOSIZE_INPUT_CLS : undefined })),
        React__default.createElement(StyledSizer, { ref: sizerRef }, inputValue)));
}));
AutosizeInput.displayName = 'AutosizeInput';

const StyledLoadingDots = styled__default.div `
  display: flex;
  align-self: center;
  text-align: center;
  margin-right: 0.25rem;
  padding: ${({ theme }) => theme.loader.padding};

  > div {
    border-radius: 100%;
    display: inline-block;

    ${bounceAnimationCss}
    width: ${({ theme }) => theme.loader.size};
    height: ${({ theme }) => theme.loader.size};
    opacity: ${({ theme }) => theme.loader.opacity};
    background-color: ${({ theme }) => theme.loader.color};

    :nth-of-type(1) {
      animation-delay: -0.272s;
    }

    :nth-of-type(2) {
      animation-delay: -0.136s;
    }
  }
`;
const LoadingDots = () => (React__default.createElement(StyledLoadingDots, { "aria-hidden": 'true' },
    React__default.createElement("div", null),
    React__default.createElement("div", null),
    React__default.createElement("div", null)));

const IndicatorIconsWrapper = styled__default.div `
  display: flex;
  flex-shrink: 0;
  align-items: center;
  align-self: stretch;
  box-sizing: border-box;
`;
const IndicatorIcon = styled__default.div `
  display: flex;
  box-sizing: border-box;
  color: ${({ theme }) => theme.icon.color};
  padding: ${({ theme }) => theme.icon.padding};

  :hover {
    color: ${({ theme }) => theme.icon.hoverColor};
  }
`;
const Clear = styled__default.div `
  overflow: hidden;
  ${fadeInAnimationCss}
  font-size: ${({ theme }) => theme.icon.clear.fontSize};
  transition: ${({ theme }) => theme.icon.clear.transition};
  font-weight: ${({ theme }) => theme.icon.clear.fontWeight};
  font-family: ${({ theme }) => theme.icon.clear.fontFamily};
`;
const Caret = styled__default.div `
  transition: ${({ theme }) => theme.icon.caret.transition};
  border-top: ${({ theme }) => theme.icon.caret.size} dashed;
  border-left: ${({ theme }) => theme.icon.caret.size} solid transparent;
  border-right: ${({ theme }) => theme.icon.caret.size} solid transparent;

  ${({ menuOpen, isInvalid, theme: { color } }) => menuOpen &&
    styled.css `
      transform: rotate(180deg);
      color: ${isInvalid ? color.invalid : (color.caretActive || color.accent)};
    `}
`;
const Separator = styled__default.div `
  width: 1px;
  margin: 0.5rem 0;
  align-self: stretch;
  box-sizing: border-box;
  background-color: ${({ theme: { color } }) => color.iconSeparator || color.border};
`;
const IndicatorIcons = React__default.memo(({ menuOpen, clearIcon, caretIcon, isInvalid, showClear, isLoading, addClassNames, onClearMouseDown, }) => {
    const clearClassName = (addClassNames && !clearIcon) ? CLEAR_ICON_CLS : undefined;
    const caretClassName = (addClassNames && !caretIcon) ? CARET_ICON_CLS : undefined;
    return (React__default.createElement(IndicatorIconsWrapper, null,
        (showClear && !isLoading) && (React__default.createElement(IndicatorIcon, { "aria-hidden": 'true', onTouchEnd: onClearMouseDown, onMouseDown: onClearMouseDown, "data-testid": CLEAR_ICON_TESTID }, clearIcon || React__default.createElement(Clear, { className: clearClassName }, "X"))),
        isLoading && React__default.createElement(LoadingDots, null),
        React__default.createElement(Separator, null),
        React__default.createElement(IndicatorIcon, { "aria-hidden": 'true' }, caretIcon || React__default.createElement(Caret, { menuOpen: menuOpen, isInvalid: isInvalid, className: caretClassName }))));
});
IndicatorIcons.displayName = 'IndicatorIcons';

const A11yText = styled__default.span `
  border: 0;
  padding: 0;
  width: 1px;
  height: 1px;
  margin: -1px;
  overflow: hidden;
  position: absolute;
  white-space: nowrap;
  clip: rect(0 0 0 0);
  clip-path: inset(50%);
`;
const AriaLiveRegion = ({ menuOpen, ariaLabel, inputValue, optionCount, isSearchable, focusedOptionLabel, focusedOptionIndex, selectedOptionLabel, isFocusedOptionDisabled, }) => (React__default.createElement(A11yText, { "aria-live": 'polite' },
    React__default.createElement("p", null, `Selected option: ${selectedOptionLabel || 'N/A'}`),
    React__default.createElement("p", null, `Focused option: ${focusedOptionLabel || 'N/A'}${isFocusedOptionDisabled ? ' - disabled' : ''}, ${focusedOptionIndex + 1} of ${optionCount}.
        ${optionCount} result${optionCount !== 1 ? 's' : ''} available${inputValue ? (' for search input ' + inputValue) : ''}.
        ${menuOpen
        ? 'Use Up and Down arrow keys to choose options, press Enter or Tab to select the currently focused option, press Escape to close the menu.'
        : `${ariaLabel || 'Select'} is focused${isSearchable ? ', type to filter options' : ''}, press Down arrow key to open the menu.`}`)));

const _indexPositionEnum = {
    UP: 0,
    DOWN: 1,
    FIRST: 2,
    LAST: 3,
};
const SelectWrapper = styled__default.div `
  position: relative;
  box-sizing: border-box;
  ${({ theme }) => theme.color.textColor && styled.css `color: ${theme.color.textColor};`}
  ${({ theme }) => theme.select.fontSize && styled.css `font-size: ${theme.select.fontSize};`}
`;
const ValueWrapper = styled__default.div `
  flex: 1 1 0%;
  display: flex;
  flex-wrap: wrap;
  overflow: hidden;
  position: relative;
  align-items: center;
  box-sizing: border-box;
  padding: ${({ theme }) => theme.control.padding};
`;
const ControlWrapper = styled__default.div `
  outline: 0;
  display: flex;
  flex-wrap: wrap;
  cursor: default;
  position: relative;
  align-items: center;
  box-sizing: border-box;
  justify-content: space-between;

  ${({ theme: { control } }) => styled.css `
    height: ${control.height};
    transition: ${control.transition};
    border-style: ${control.borderStyle};
    border-width: ${control.borderWidth};
    border-radius: ${control.borderRadius};
    ${control.backgroundColor && `background-color: ${control.backgroundColor};`}
  `}

  ${({ isDisabled, theme }) => isDisabled &&
    styled.css `
      pointer-events: none;
      background-color: ${theme.color.disabled};
    `}

  ${({ isFocused, isInvalid, theme }) => renderControlEmphasis(theme.control.boxShadow, theme.color.border, theme.color.invalid, theme.color.invalidFocus, theme.control.boxShadowColor, theme.control.focusedBorderColor, isFocused, isInvalid)}
`;
const MenuWrapper = styled__default.div `
  z-index: 999;
  position: absolute;
  ${fadeInAnimationCss}
  
  ${({ hideMenu, theme: { menu } }) => styled.css `
    width: ${menu.width};
    margin: ${menu.margin};
    padding: ${menu.padding};
    box-shadow: ${menu.boxShadow};
    border-radius: ${menu.borderRadius};
    background-color: ${menu.backgroundColor};
    ${hideMenu && 'display: none;'}
  `}

  .${OPTION_CLS} {
    display: block;
    overflow: hidden;
    user-select: none;
    white-space: nowrap;
    text-overflow: ellipsis;
    -webkit-tap-highlight-color: rgba(0, 0, 0, 0);
    padding: ${({ theme }) => theme.menu.option.padding};
    text-align: ${({ theme }) => theme.menu.option.textAlign};

    &.${OPTION_FOCUSED_CLS},
    &:hover:not(.${OPTION_DISABLED_CLS}):not(.${OPTION_SELECTED_CLS}) {
      background-color: ${({ theme }) => theme.menu.option.focusedBgColor};
    }

    &.${OPTION_SELECTED_CLS} {
      color: ${({ theme }) => theme.menu.option.selectedColor};
      background-color: ${({ theme }) => theme.menu.option.selectedBgColor};
    }

    &.${OPTION_DISABLED_CLS} {
      opacity: 0.35;
    }
  }
`;
const Select = React__default.forwardRef(({ inputId, selectId, idSuffix, isLoading, onKeyDown, clearIcon, caretIcon, isInvalid, ariaLabel, menuWidth, isDisabled, inputDelay, onMenuOpen, onMenuClose, onInputBlur, isClearable, onInputFocus, initialValue, addClassNames, ariaLabelledBy, onOptionChange, getOptionLabel, getOptionValue, openMenuOnFocus, isAriaLiveEnabled, menuOverscanCount, blurInputOnSelect, renderOptionLabel, getIsOptionDisabled, filterIsCaseSensitive, getFilterOptionString, isSearchable = true, tabSelectsOption = true, closeMenuOnSelect = true, scrollMenuIntoView = true, backspaceClearsValue = true, options = OPTIONS_DEFAULT, placeholder = PLACEHOLDER_DEFAULT, noOptionsMsg = NO_OPTIONS_MSG_DEFAULT, menuItemSize = MENU_ITEM_SIZE_DEFAULT, menuMaxHeight = MENU_MAX_HEIGHT_DEFAULT, }, ref) => {
    const listRef = React.useRef(null);
    const menuRef = React.useRef(null);
    const inputRef = React.useRef(null);
    const [inputValue, setInputValue] = React.useState('');
    const [menuOpen, setMenuOpen] = React.useState(false);
    const [isFocused, setIsFocused] = React.useState(false);
    const [focusedOption, setFocusedOption] = React.useState(FOCUSED_OPTION_DEFAULT);
    const [selectedOption, setSelectedOption] = React.useState(SELECTED_OPTION_DEFAULT);
    const debouncedInputValue = useDebounce(inputValue, inputDelay);
    const { data: selectedOptionData, value: selectedOptionValue, label: selectedOptionLabel, initFlag: canAttemptInitValSet, } = selectedOption;
    const { data: focusedOptionData, value: focusedOptionValue, label: focusedOptionLabel, index: focusedOptionIndex, isDisabled: isFocusedOptionDisabled, } = focusedOption;
    const getOptionLabelCB = React.useCallback((data) => {
        return getOptionLabel ? getOptionLabel(data) : data.label;
    }, [getOptionLabel]);
    const getOptionValueCB = React.useCallback((data) => {
        return getOptionValue ? getOptionValue(data) : data.value;
    }, [getOptionValue]);
    const getIsOptionDisabledCB = React.useCallback((data) => {
        return getIsOptionDisabled ? getIsOptionDisabled(data) : !!data.isDisabled;
    }, [getIsOptionDisabled]);
    const renderOptionLabelCB = React.useCallback((data) => {
        return renderOptionLabel ? renderOptionLabel(data) : getOptionLabelCB(data);
    }, [renderOptionLabel, getOptionLabelCB]);
    const getFilterOptionStringCB = React.useCallback((option) => {
        return getFilterOptionString ? getFilterOptionString(option) : String(option.label);
    }, [getFilterOptionString]);
    const menuOptions = useMenuOptions(options, debouncedInputValue, getOptionValueCB, getOptionLabelCB, getIsOptionDisabledCB, getFilterOptionStringCB, filterIsCaseSensitive);
    const blurInput = () => {
        inputRef.current && inputRef.current.blur();
    };
    const focusInput = () => {
        inputRef.current && inputRef.current.focus();
    };
    const scrollToItemIndex = (index) => {
        listRef.current && listRef.current.scrollToItem(index);
    };
    const selectOption = React.useCallback((option, isSelected) => {
        if (isSelected) {
            setMenuOpen(false);
        }
        else {
            setSelectedOption(option || NO_SELECTED_OPTION);
        }
    }, []);
    const openMenuAndFocusOption = React.useCallback((position) => {
        if (!isArrayWithLength(menuOptions)) {
            setMenuOpen(true);
            return;
        }
        const selectedIndex = selectedOption.value
            ? menuOptions.findIndex((option) => option.value === selectedOption.value)
            : -1;
        const index = (selectedIndex > -1)
            ? selectedIndex : (position === _indexPositionEnum.FIRST) ? 0
            : menuOptions.length - 1;
        setMenuOpen(true);
        setFocusedOption(Object.assign({ index }, menuOptions[index]));
        scrollToItemIndex(index);
    }, [menuOptions, selectedOption]);
    React.useImperativeHandle(ref, () => ({
        blur: blurInput,
        focus: focusInput,
        clearValue: () => {
            selectOption();
            setFocusedOption(FOCUSED_OPTION_DEFAULT);
        },
        setValue: (option) => {
            const validOptionOrUndefined = validateSetValueOption(option, menuOptions, getOptionValueCB);
            selectOption(validOptionOrUndefined);
        },
    }));
    React.useEffect(() => {
        if (isFocused && openMenuOnFocus) {
            openMenuAndFocusOption(_indexPositionEnum.FIRST);
        }
    }, [isFocused, openMenuOnFocus, openMenuAndFocusOption]);
    React.useEffect(() => {
        if (canAttemptInitValSet && isPlainObject(initialValue)) {
            selectOption({
                data: initialValue,
                value: getOptionValueCB(initialValue),
                label: getOptionLabelCB(initialValue),
            });
        }
    }, [selectOption, initialValue, canAttemptInitValSet, getOptionValueCB, getOptionLabelCB]);
    React.useEffect(() => {
        onOptionChange && onOptionChange(selectedOptionData || null);
        !selectedOptionData && setFocusedOption(FOCUSED_OPTION_DEFAULT);
        if (blurInputOnSelect && selectedOptionData) {
            blurInput();
        }
        else if (closeMenuOnSelect) {
            setMenuOpen(false);
            setInputValue('');
        }
    }, [selectedOptionData, onOptionChange, closeMenuOnSelect, blurInputOnSelect]);
    React.useEffect(() => {
        if (menuOpen) {
            scrollMenuIntoViewOnOpen(menuRef.current, scrollMenuIntoView, onMenuOpen);
        }
        else {
            onMenuClose && onMenuClose();
        }
    }, [menuOpen, onMenuClose, onMenuOpen, scrollMenuIntoView]);
    React.useEffect(() => {
        if (menuOptions.length === 1 || (!!menuOptions.length && menuOptions.length !== options.length)) {
            setFocusedOption(Object.assign({ index: 0 }, menuOptions[0]));
            scrollToItemIndex(0);
        }
    }, [options, menuOptions]);
    const selectOptionFromFocused = () => {
        if (!focusedOptionData || isFocusedOptionDisabled) {
            return;
        }
        if (closeMenuOnSelect && (selectedOptionValue === focusedOptionValue)) {
            setMenuOpen(false);
        }
        else {
            selectOption({
                data: focusedOptionData,
                value: focusedOptionValue,
                label: focusedOptionLabel,
            });
        }
    };
    const focusOptionOnArrowKey = (position) => {
        if (!isArrayWithLength(menuOptions)) {
            return;
        }
        const index = (position === _indexPositionEnum.DOWN)
            ? (focusedOptionIndex + 1) % menuOptions.length
            : (focusedOptionIndex > 0)
                ? (focusedOptionIndex - 1)
                : (menuOptions.length - 1);
        const nextFocusedOption = menuOptions[index];
        if (nextFocusedOption) {
            setFocusedOption(Object.assign({ index }, nextFocusedOption));
            scrollToItemIndex(index);
        }
    };
    const handleOnKeyDown = (e) => {
        if (isDisabled)
            return;
        if (onKeyDown) {
            onKeyDown(e);
            if (e.defaultPrevented)
                return;
        }
        switch (e.key) {
            case 'ArrowDown':
                if (menuOpen) {
                    focusOptionOnArrowKey(_indexPositionEnum.DOWN);
                }
                else {
                    openMenuAndFocusOption(_indexPositionEnum.FIRST);
                }
                break;
            case 'ArrowUp':
                if (menuOpen) {
                    focusOptionOnArrowKey(_indexPositionEnum.UP);
                }
                else {
                    openMenuAndFocusOption(_indexPositionEnum.LAST);
                }
                break;
            case ' ':
                if (inputValue) {
                    return;
                }
                else if (!menuOpen) {
                    openMenuAndFocusOption(_indexPositionEnum.FIRST);
                }
                else {
                    selectOptionFromFocused();
                }
                break;
            case 'Enter':
                if (e.keyCode !== 229) {
                    selectOptionFromFocused();
                }
                break;
            case 'Escape':
                setMenuOpen(false);
                setInputValue('');
                break;
            case 'Tab':
                if (e.shiftKey || !menuOpen || !focusedOptionData) {
                    return;
                }
                else if (tabSelectsOption) {
                    selectOptionFromFocused();
                }
                else {
                    setMenuOpen(false);
                    setInputValue('');
                }
                break;
            case 'Delete':
            case 'Backspace':
                if (inputValue || !backspaceClearsValue || !isClearable) {
                    return;
                }
                selectOption();
                break;
            default:
                return;
        }
        e.preventDefault();
    };
    const handleOnControlMouseDown = (e) => {
        if (isDisabled)
            return;
        if (!isFocused)
            focusInput();
        const targetIsNotInput = (e.currentTarget.tagName !== 'INPUT');
        if (!menuOpen) {
            openMenuAndFocusOption(_indexPositionEnum.FIRST);
        }
        else if (targetIsNotInput) {
            setMenuOpen(false);
            inputValue && setInputValue('');
        }
        if (targetIsNotInput) {
            e.preventDefault();
        }
    };
    const handleOnMenuMouseDown = (e) => {
        e.stopPropagation();
        e.preventDefault();
        focusInput();
    };
    const handleOnInputBlur = React.useCallback((e) => {
        onInputBlur && onInputBlur(e);
        setIsFocused(false);
        setMenuOpen(false);
        setInputValue('');
    }, [onInputBlur]);
    const handleOnInputFocus = React.useCallback((e) => {
        onInputFocus && onInputFocus(e);
        setIsFocused(true);
    }, [onInputFocus]);
    const handleOnInputChange = React.useCallback((e) => {
        setMenuOpen(true);
        setInputValue(e.currentTarget.value || '');
    }, []);
    const handleOnClearMouseDown = React.useCallback((e) => {
        e.stopPropagation();
        (e.type === 'mousedown') && e.preventDefault();
        setSelectedOption(NO_SELECTED_OPTION);
        focusInput();
    }, []);
    return (React__default.createElement(SelectWrapper, { id: selectId, onKeyDown: handleOnKeyDown, "data-testid": SELECT_CONTAINER_TESTID, className: addClassNames ? SELECT_CONTAINER_CLS : undefined },
        React__default.createElement(ControlWrapper, { isInvalid: isInvalid, isFocused: isFocused, isDisabled: isDisabled, onTouchEnd: handleOnControlMouseDown, onMouseDown: handleOnControlMouseDown, "data-testid": CONTROL_CONTAINER_TESTID, className: addClassNames ? CONTROL_CONTAINER_CLS : undefined },
            React__default.createElement(ValueWrapper, null,
                React__default.createElement(Value, { inputValue: inputValue, placeholder: placeholder, selectedOptionData: selectedOptionData, renderOptionLabel: renderOptionLabelCB }),
                React__default.createElement(AutosizeInput, { id: inputId, ref: inputRef, disabled: isDisabled, ariaLabel: ariaLabel, inputValue: inputValue, onBlur: handleOnInputBlur, isSearchable: isSearchable, onFocus: handleOnInputFocus, addClassNames: addClassNames, onChange: handleOnInputChange, ariaLabelledBy: ariaLabelledBy })),
            React__default.createElement(IndicatorIcons, { menuOpen: menuOpen, clearIcon: clearIcon, caretIcon: caretIcon, isInvalid: isInvalid, isLoading: isLoading, addClassNames: addClassNames, onClearMouseDown: handleOnClearMouseDown, showClear: !!(isClearable && !isDisabled && selectedOptionData) })),
        React__default.createElement(MenuWrapper, { ref: menuRef, hideMenu: !menuOpen, onMouseDown: handleOnMenuMouseDown, "data-testid": MENU_CONTAINER_TESTID, className: addClassNames ? MENU_CONTAINER_CLS : undefined },
            React__default.createElement(Menu, { ref: listRef, width: menuWidth, idSuffix: idSuffix, itemSize: menuItemSize, maxHeight: menuMaxHeight, menuOptions: menuOptions, noOptionsMsg: noOptionsMsg, selectOption: selectOption, overscanCount: menuOverscanCount, focusedOptionIndex: focusedOptionIndex, renderOptionLabel: renderOptionLabelCB, selectedOptionValue: selectedOptionValue })),
        (isAriaLiveEnabled && isFocused) && (React__default.createElement(AriaLiveRegion, { menuOpen: menuOpen, ariaLabel: ariaLabel, inputValue: inputValue, isSearchable: isSearchable, optionCount: menuOptions.length, focusedOptionLabel: focusedOptionLabel, focusedOptionIndex: focusedOptionIndex, selectedOptionLabel: selectedOptionLabel, isFocusedOptionDisabled: isFocusedOptionDisabled }))));
});
Select.displayName = 'Select';

const SelectWrapper$1 = React__default.forwardRef((_a, ref) => {
    var { themeConfig } = _a, props = __rest(_a, ["themeConfig"]);
    const theme = useTheme(themeConfig);
    const menuWidth = props.menuWidth || theme.menu.width;
    const blurInputOnSelect = (typeof props.blurInputOnSelect === 'boolean')
        ? props.blurInputOnSelect
        : isTouchDevice();
    const selectProps = Object.assign(Object.assign({}, props), { menuWidth,
        blurInputOnSelect, inputId: createID(props.inputId, props.idSuffix), selectId: createID(props.selectId, props.idSuffix) });
    return (React__default.createElement(styled.ThemeProvider, { theme: theme },
        React__default.createElement(Select, Object.assign({ ref: ref }, selectProps))));
});
SelectWrapper$1.displayName = 'SelectWrapper';

exports.Select = SelectWrapper$1;
